#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include "SDL.h"
#include "libimg/image.h"

#define XSZ		640
#define YSZ		480

int init(void);
void redraw(unsigned long msec);
void clean_up(void);

SDL_Surface *fbsurf;

int x = 10, y = 10;
unsigned long imgx, imgy;
uint32_t *img;

int main(int argc, char **argv) {
	unsigned long start_time;
	
	SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
	if(!(fbsurf = SDL_SetVideoMode(XSZ, YSZ, 32, SDL_SWSURFACE))) {
		fprintf(stderr, "failed to initialize SDL\n");
		return EXIT_FAILURE;
	}
	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL);
	
	if(init() == -1) {
		return EXIT_FAILURE;
	}
	start_time = SDL_GetTicks();
	
	for(;;) {
		SDL_Event event;
		while(SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym) {
				case SDLK_ESCAPE:
					clean_up();
					return 0;
					
				case SDLK_LEFT:
					x -= 5;
					if(x < 0) {
						x = 0;
					}
					break;
					
				case SDLK_RIGHT:
					x += 5;
					if(x >= XSZ - imgx) {
						x = XSZ - imgx;
					}
					break;
					
				case SDLK_UP:
					y -= 5;
					if(y < 0) {
						y = 0;
					}
					break;
					
				case SDLK_DOWN:
					y += 5;
					if(y >= YSZ - imgy) {
						y = YSZ - imgy;
					}
					break;
					
				default:
					break;
				}
				break;
				
			default:
				break;
			}
		}
		redraw(SDL_GetTicks() - start_time);
	}
	
	return 0;
}

int init(void) {
	if(!(img = load_image("data/90.png", &imgx, &imgy))) {
		return -1;
	}
	
	return 0;
}

void redraw(unsigned long msec) {
	int i;
	uint32_t *fb = fbsurf->pixels;
	
	memset(fb, 0xff, XSZ * YSZ * sizeof(uint32_t));
	
	for(i=0; i<imgy; i++) {
		memcpy(fb + y * XSZ + x + i * XSZ, img + i * imgx, imgx * sizeof(uint32_t));
	}
	
	SDL_Flip(fbsurf);
}

void clean_up(void) {
	SDL_Quit();
}
